#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import datetime

def test_name(rec):
    assert rec.locus["name"] == "BRRBIF72"
    assert rec.locus["date"] == datetime.date(1996, 3, 1)

def test_keyword(rec):
    assert rec.keywords["keywords"] == []

def test_feature(rec):
    assert rec.features[0]["properties"]["dev_stage"] == "flower" #power

def test_counts(rec):
    assert rec.base_count == {"A": 88, "C": 56, "G": 80, "T": 58}

def test_sequence(rec):
    data = """
        aacaaaactc aataaataaa caaatggcag acaacaagca gagcttccaa gccggtcaag
        ccgctggtcg tgctgaggag aagggtaatg tgctgctgat ggacaaggtc aaggatgctg
        ctaccgcagc tggagctctc caaaccgcgg gacagaagat aacggaggcg gcagggggag
        ccgttaatct cgtgaaggag aagaccggca tgaacaagta gccccattgg aaacaaaatt
        gggagttata gtttcctttt taatattaat cgttgtggtt tt
    """
    assert ''.join(s for s in rec.seqiter) == ''.join(data.split())



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "SCX3_BUTOC"

def test_keywords(rec):
    assert rec.keywords["keywords"] == [
        "Neurotoxin",
        "Sodium channel inhibitor",
        "Amidation"
    ]


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import t

@t.parse("parse_location")
def test_index_1():
    "123"
    return {"position": 123, "fuzzy": False, "type": "index"}

@t.parse("parse_location")
def test_index_2():
    "<34"
    return {"position": 34, "fuzzy": True, "type": "index"}

@t.parse("parse_location")
def test_index_3():
    ">935345"
    return {"position": 935345, "fuzzy": True, "type": "index"}

@t.parse("parse_location")
def test_bond():
    "bond(1,5)"
    return {"type": "bond", "positions": [1, 5]}

@t.parse("parse_location")
def test_one_of():
    "one-of(89,453)"
    return {"type": "one-of", "positions": [89, 453]}

@t.parse("parse_location")
def test_span_1():
    "1..30"
    return {"type": "span", "from": 1, "to": 30}

@t.parse("parse_location")
def test_span_2():
    "<1..>45"
    return {
        "type": "span",
        "from": 1,
        "to": 45,
        "modifiers": {"from": "fuzzy", "to": "fuzzy"}
    }

@t.parse("parse_location")
def test_span_3():
    "one-of(8,9)..56"
    return {
        "type": "span",
        "from": {"type": "one-of", "positions": [8, 9]},
        "to": 56
    }

@t.parse("parse_location")
def test_site():
    "45^46"
    return {
        "type": "site",
        "between": [
            {"type": "index", "position": 45, "fuzzy": False},
            {"type": "index", "position": 46, "fuzzy": False}
        ]
    }

@t.parse("parse_location")
def test_choice():
    "90.414"
    return {
        "type": "choice",
        "between": [
            {"type": "index", "position": 90, "fuzzy": False},
            {"type": "index", "position": 414, "fuzzy": False}
        ]
    }

@t.parse("parse_location")
def test_gap():
    "gap(100)"
    return {"type": "gap", "length": 100}

@t.parse("parse_location")
def test_join_1():
    "join(5..10,25..45)"
    return {
        "type": "join",
        "segments": [
            {"type": "span", "from": 5, "to": 10},
            {"type": "span", "from": 25, "to": 45}
        ]
    }

@t.parse("parse_location")
def test_join_2():
    "join(14..24,complement(56..34))"
    return {
        "type": "join",
        "segments": [
            {"type": "span", "from": 14, "to": 24},
            {
                "type": "complement",
                "segment": {"type": "span", "from": 56, "to": 34}
            }
        ]
    }

@t.parse("parse_location")
def test_complement_1():
    "complement(15..34)"
    return {
        "type": "complement",
        "segment": {"type": "span", "from": 15, "to": 34}
    }

@t.parse("parse_location")
def test_complement_2():
    "complement(join(45..90,100..102))"
    return {
        "type": "complement",
        "segment": {
            "type": "join",
            "segments": [
                {"type": "span", "from": 45, "to": 90},
                {"type": "span", "from": 100, "to": 102}
            ]
        }
    }

@t.parse("parse_location")
def test_order():
    "order(4..5,9..10)"
    return {
        "type": "order",
        "segments": [
            {"type": "span", "from": 4, "to": 5},
            {"type": "span", "from": 9, "to": 10}
        ]
    }

@t.parse("parse_location")
def test_reference():
    "NC_001912.5:5..9"
    return {
        "type": "reference",
        "accession": "NC_001912.5",
        "segment": {"type": "span", "from": 5, "to": 9}
    }




#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "SCX3_BUTOC"

def test_description(rec):
    assert rec.keywords["definition"] == "Neurotoxin III"

def test_pid(rec):
    assert rec.keywords["pid"] == "g134354"

def test_kw_length(rec):
    assert len(rec.keywords) == 9
    assert len(rec.keywords["reference"]) == 1

def test_feature_length(rec):
    assert len(rec.features) == 7
    
def test_feature_prop_lengths(rec):
    data = [2, 1, 2, 2, 2, 2, 1]
    for r, e in zip(rec.features, data):
        assert len(r["properties"]) == e

def test_bond_loc(rec):
    assert rec.features[2]["location"] == {
        "type": "bond",
        "positions": [12, 63]
    }

def test_site_index(rec):
    assert rec.features[6]["type"] == "Site"
    assert rec.features[6]["location"] == {
        "type": "index",
        "position": 64,
        "fuzzy": False
    }



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "ATCOR66M"

def test_keyword(rec):
    assert rec.keywords["reference"][1]["medline"] == "92119220"

def test_feature(rec):
    assert rec.features[2]["properties"]["translation"] == "".join("""
        MSETNKNAFQAGQAAGKAEEKSNVLLDKAKDAAAAAGASAQQAG
        KSISDAAVGGVNFVKDKTGLNK
    """.split())

def test_counts(rec):
    assert rec.base_count == {"A": 194, "C": 82, "G": 104, "T": 133}

def test_sequence(rec):
    data = """
        aacaaaacac acatcaaaaa cgattttaca agaaaaaaat atctgaaaaa tgtcagagac
        caacaagaat gccttccaag ccggtcaggc cgctggcaaa gctgaggaga agagcaatgt
        tctgctggac aaggccaagg atgctgctgc tgcagctgga gcttccgcgc aacaggcggg
        aaagagtata tcggatgcgg cagtgggagg tgttaacttc gtgaaggaca agaccggcct
        gaacaagtag cgatccgagt caactttggg agttataatt tcccttttct aattaattgt
        tgggattttc aaataaaatt tgggagtcat aattgattct cgtactcatc gtacttgttg
        ttgtttttag tgttgtaatg ttttaatgtt tcttctccct ttagatgtac tacgtttgga
        actttaagtt taatcaacaa aatctagttt aagttctaaa aaaaaaaaaa aaaaaaaaaa
        aaaaaaaaaa aaaaaaaaaa aaaaaaaaaa aaa
    """
    assert ''.join(s for s in rec.seqiter) == ''.join(data.split())



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import t

def test_1():
    data = """
        1 cacaggccca gagccactcc tgcctacagg ttctgagggc tcaggggacc tcctgggccc
       61 tcaggctctt tagctgagaa taagggccct gagggaacta cctgcttctc acatccccgg
      121 gtctctgacc atctgctgtg tgccccgacc ccccctaccc tgctcctcca ccaagcctga
      181 tgccaagggc tataaaccac tggcccaaca gaagcttggt tcccagagaa ctggtccctg
      241 cctgggacat gctccttgct acagcccctt gtgggagctc agagggcatg gctgctcccc
      301 ctacggtccc tcgcccagtg gttctgtctc tttatggcag gaagcaatga ggctccccaa
      361 gaacacacct gaggaaaagg acaggtgagc ctggagggcc ggccgcaccg tgggcctctg
      421 tgtctgggga gttggtggcc aggatcccga gtacctgggt gctgtgacgg ggcgtggttg
      481 gcctgggcgt gctgggtgtt tgggaatgac ttcccatcgc ccgcttctgc agcctgctca
      541 gccctgttgg ggtgcagtgt gtccactgcc tgcctgtgtg tgccgctgtg ctcaggctct
      601 cctcttgctc ctttcaggcg cacggcggcc ctacaggagg gtctgaggcg ggcagtctct
      661 gtgccgctga cgctggcgga gacggtggcc tcgctgtggc cggccctgca ggaactggcc
      721 cggtgtggga acctggcctg ccggtcagac ctccaggtag gggggcccgg gggaccccag
      781 gcctcctgcc gcaaagcagg aagcagctgt tggggctgag ttgctggaga gcacggtggt
      841 tggctgggct gagccaggta gggagacctc acttcacggg cagttcccgg ggctttggcc
      901 tcctccaaca gggccggggt gtccgctgcc ttctaagatc tcgctcacgg cggcaccaca
      961 gacggagacc caaatgtgtc tgcagagccc accctgacca ctgtataagt gtatactccc
     1021 ccaagacccc ttgtatcacc cccactgtcg tgttctagaa aacacctatg tcagcccagg
     1081 cacaatggtt ctcgcctgtg gtcccagcac tttgggaggc tgaggcggga ggatcacttg
     1141 aatagaggag gtcgagacca gcctgggcaa catagtggga ccctggctgt acaatagata
     1201 catatgagcc aggcacggtg gtgcctgtgg tcccagctcc ctgcccagcc agcctcctgt
     1261 ctccagagaa gttctccatt aaaaaataat ttagcaaaaa aaaaaaaaaa aaaaaaaaaa
     1321 aaaaaa
//
    """.lstrip("\n")
    reader = t.nebgb.Reader.from_string(data)
    seqiter = t.nebgb.Sequence(reader)
    expiter = iter([
        "cacaggcccagagccactcctgcctacaggttctgagggctcaggggacctcctgggccc",
        "tcaggctctttagctgagaataagggccctgagggaactacctgcttctcacatccccgg",
        "gtctctgaccatctgctgtgtgccccgaccccccctaccctgctcctccaccaagcctga",
        "tgccaagggctataaaccactggcccaacagaagcttggttcccagagaactggtccctg",
        "cctgggacatgctccttgctacagccccttgtgggagctcagagggcatggctgctcccc",
        "ctacggtccctcgcccagtggttctgtctctttatggcaggaagcaatgaggctccccaa",
        "gaacacacctgaggaaaaggacaggtgagcctggagggccggccgcaccgtgggcctctg",
        "tgtctggggagttggtggccaggatcccgagtacctgggtgctgtgacggggcgtggttg",
        "gcctgggcgtgctgggtgtttgggaatgacttcccatcgcccgcttctgcagcctgctca",
        "gccctgttggggtgcagtgtgtccactgcctgcctgtgtgtgccgctgtgctcaggctct",
        "cctcttgctcctttcaggcgcacggcggccctacaggagggtctgaggcgggcagtctct",
        "gtgccgctgacgctggcggagacggtggcctcgctgtggccggccctgcaggaactggcc",
        "cggtgtgggaacctggcctgccggtcagacctccaggtaggggggcccgggggaccccag",
        "gcctcctgccgcaaagcaggaagcagctgttggggctgagttgctggagagcacggtggt",
        "tggctgggctgagccaggtagggagacctcacttcacgggcagttcccggggctttggcc",
        "tcctccaacagggccggggtgtccgctgccttctaagatctcgctcacggcggcaccaca",
        "gacggagacccaaatgtgtctgcagagcccaccctgaccactgtataagtgtatactccc",
        "ccaagaccccttgtatcacccccactgtcgtgttctagaaaacacctatgtcagcccagg",
        "cacaatggttctcgcctgtggtcccagcactttgggaggctgaggcgggaggatcacttg",
        "aatagaggaggtcgagaccagcctgggcaacatagtgggaccctggctgtacaatagata",
        "catatgagccaggcacggtggtgcctgtggtcccagctccctgcccagccagcctcctgt",
        "ctccagagaagttctccattaaaaaataatttagcaaaaaaaaaaaaaaaaaaaaaaaaa",
        "aaaaaa"
    ])
    t.samesequence(seqiter, expiter)

def test_2():
    data = """
        1 meecwvteia ngskdgldsn pmkdymilsg pqktavavlc tllgllsale nvavlylils
       61 shqlrrkpsy lfigslagad flasvvfacs fvnfhvfhgv dskavfllki gsvtmtftas
      121 vgsllltaid rylclrypps ykalltrgra lvtlgimwvl salvsylplm gwtccprpcs
      181 elfplipndy llswllfiaf lfsgiiytyg hvlwkahqhv aslsghqdrq vpgmarmrld
      241 vrlaktlglv lavllicwfp vlalmahsla ttlsdqvkka fafcsmlcli nsmvnpviya
      301 lrsgeirssa hhclahwkkc vrglgseake eaprssvtet eadgkitpwp dsrdldlsdc
      361 
//
    """.lstrip("\n")
    reader = t.nebgb.Reader.from_string(data)
    seqiter = t.nebgb.Sequence(reader)
    expiter = iter([
        "meecwvteiangskdgldsnpmkdymilsgpqktavavlctllgllsalenvavlylils",
        "shqlrrkpsylfigslagadflasvvfacsfvnfhvfhgvdskavfllkigsvtmtftas",
        "vgsllltaidrylclryppsykalltrgralvtlgimwvlsalvsylplmgwtccprpcs",
        "elfplipndyllswllfiaflfsgiiytyghvlwkahqhvaslsghqdrqvpgmarmrld",
        "vrlaktlglvlavllicwfpvlalmahslattlsdqvkkafafcsmlclinsmvnpviya",
        "lrsgeirssahhclahwkkcvrglgseakeeaprssvteteadgkitpwpdsrdldlsdc"
    ])
    t.samesequence(seqiter, expiter)
    



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import t

@t.parse("parse_locus")
def test_1():
    """
LOCUS       DMBR25B3   154329 bp    DNA             INV       07-FEB-2000
    """
    return {
        'name': 'DMBR25B3',
        'length': 154329,
        'base_type': 'bp',
        'molecule_type': 'DNA',
        'division': 'INV',
        'date': t.datetime.date(2000, 2, 7)
    }

@t.parse("parse_locus")
def test_2():
    """
LOCUS       EU032008                1005 bp    RNA     linear   VRL 30-SEP-2007    
    """
    return {
        "name": "EU032008",
        "length": 1005,
        "base_type": "bp",
        "molecule_type": "RNA",
        "topology": "linear",
        "division": "VRL",
        "date": t.datetime.date(2007, 9, 30)
    }
    
@t.parse("parse_locus")
def test_3():
    """
LOCUS       NM_015658               2817 bp    mRNA    linear   PRI 27-SEP-2007
    """
    return {
        'name': 'NM_015658',
        'length': 2817,
        'base_type': 'bp',
        'molecule_type': 'mRNA',
        'topology': 'linear',
        'division': 'PRI',
        'date': t.datetime.date(2007, 9, 27)
    }
    
@t.parse("parse_locus")
def test_4():
    """
LOCUS       SCX3_BUTOC                64 aa            linear   INV 16-OCT-2001
    """
    return {
        'name': 'SCX3_BUTOC',
        'length': 64,
        'base_type': 'aa',
        'topology': 'linear',
        'division': 'INV',
        'date': t.datetime.date(2001, 10, 16),
    }



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "BNAKINI"

def test_keyword(rec):
    assert rec.keywords["reference"][0]["refid"] == 1

def test_feature(rec):
    assert rec.features[5]["properties"]["gene"] == "kin1"

def test_counts(rec):
    assert rec.base_count == {"A": 129, "C": 76, "G": 110, "T": 126}

def test_sequence(rec):
    data = """
        aaaaaaacac aacaaaactc aataaataaa caaatggcag acaacaagca gagcttccaa
        gccggtcaag cctctggtcg tgctgaggag aagggtaatg tgctgatgga caaggtcaag
        gatgctgcta ccgcagctgg agcgtctgcg caaaccgcgg gacagaagat aacggaggcg
        gcagggggag ccgttaatct cgtgaaggag aagaccggca tgaacaagta gccccattgg
        aaataaaatt gggagttata gtttcccttt ttaatgttaa tcgttgtggt tttaaataaa
        aattgggtgt tctaattgat cttcaccgta gttgttgttg ttgttgtttt tagtgtttga
        agtaatgttt tcaaccttta ggtgtatgtc ttgatgttta tgtacttagt tcccccttaa
        tgaacatctg atttaagctt c
    """
    assert ''.join(s for s in rec.seqiter) == ''.join(data.split())


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "NT_019265"
    assert rec.locus["length"] == 1250660

def test_kw_length(rec):
    assert len(rec.keywords) == 7
    assert len(rec.keywords["reference"]) == 1

def test_feature_length(rec):
    assert len(rec.features) == 5
    
def test_feature_prop_lengths(rec):
    data = [3, 4, 2, 2, 4]
    for r, e in zip(rec.features, data):
        assert len(r["properties"]) == e

def test_feature_variation(rec):
    assert rec.features[3] == {
        "type": "variation",
        "location": {
            "type": "index",
            "position": 217508,
            "fuzzy": False
        },
        "properties": {
            "allele": ["T", "C"],
            "db_xref": ["dbSNP:811400"]
        }
    }



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "ARU237582"

def test_keyword(rec):
    assert rec.keywords["version"] == {
        "accession": "AJ237582.1",
        "gi": "4538892"
    }

def test_feature(rec):
    assert rec.features[4]["type"] == "CDS"
    assert rec.features[4]["properties"]["codon_start"] == 2

def test_counts(rec):
    assert rec.base_count == {"A": 65, "C": 38, "G": 53, "T": 48, "OTHERS": 2}

def test_sequence(rec):
    data = """
        ggacaaggcc aaggatgctg ctgctgcagc tggagcttcc gcgcaacaag taaacagata
        cacacacatg acacatatat aantacatat cacaagtagg tcgtagattt ctgatatntt
        tgtattgtgt attacgtata taggcgggaa agaacatatc ggatgcagca gctggaggtg
        ttaacttcgt gaaggagaag accggc
    """
    assert ''.join(s for s in rec.seqiter) == ''.join(data.split())



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import t

@t.parse("parse_feature")
def test_basic():
    """
    source          1..182
                    /organism="Mus musculus"
                    /db_xref="taxon:10090"
                    /chromosome="4"
                    /map="4 42.6 cM"
    """
    return {
        "type": "source",
        "location": {"type": "span", "from": 1, "to": 182},
        "properties": {
            "organism": "Mus musculus",
            "db_xref": ["taxon:10090"],
            "chromosome": "4",
            "map": "4 42.6 cM"
        }
    }
    
@t.parse("parse_feature")
def test_multiline_qualifiers():
    """
    CDS             2..3340
                    /gene="ITSN1"
                    /note="alternatively spliced; produced by truncation of
                    exon 6 and alternative splicing of exons 25 and 26"
                    /db_xref="GI:157497186"
                    /translation="MAQFPTPFGGSLDIWAITVEERAKHDQQFHSLKPISGFITGDQA
                    VNGQVGLFPSNYVKLTTDMDPSQQ"
    """
    return {
        "type": "CDS",
        "location": {"type": "span", "from": 2, "to": 3340},
        "properties": {
            "gene": "ITSN1",
            "note": t.normalize("""
                alternatively spliced; produced by truncation of
                exon 6 and alternative splicing of exons 25 and 26
            """),
            "db_xref": ["GI:157497186"],
            "translation": "".join("""
                MAQFPTPFGGSLDIWAITVEERAKHDQQFHSLKPISGFITGDQA
                VNGQVGLFPSNYVKLTTDMDPSQQ        
            """.split())
        }
    }
    
@t.parse("parse_feature")
def test_multiline_location():
    """
    CDS             complement(join(70720..70988,71424..71621,72605..72768,
                    72839..73016,73086..73559,75217..75241))
                    /gene="EG:BACR25B3.2"
    """
    return {
        "type": "CDS",
        "location": {
            "type": "complement",
            "segment": {
                "type": "join",
                "segments": [
                    {"type": "span", "from": 70720, "to": 70988},
                    {"type": "span", "from": 71424, "to": 71621},
                    {"type": "span", "from": 72605, "to": 72768},
                    {"type": "span", "from": 72839, "to": 73016},
                    {"type": "span", "from": 73086, "to": 73559},
                    {"type": "span", "from": 75217, "to": 75241}
                ]
            }
        },
        "properties": {
            "gene": "EG:BACR25B3.2",
        }
    }

@t.parse("parse_feature")
def test_qualifier_no_value():
    """
    CDS             1..360
                    /pseudo
                    /gene="CNR2"
    """
    return {
        "type": "CDS",
        "location": {"type": "span", "from": 1, "to": 360},
        "properties": {
            "pseudo": True,
            "gene": "CNR2"
        }
    }

@t.parse("parse_feature")
def test_qualifier_fake_out():
    """
    source          1..1326
                    /organism="Homo sapiens"
                    /db_xref="taxon:9606"
                    /chromosome="21"
                    /clone="IMAGE cDNA clone 125195"
                    /clone_lib="Soares fetal liver spleen 1NFLS"
                    /note="contains Alu repeat; likely to be be derived from
                    unprocessed nuclear RNA or genomic DNA; encodes putative
                    exons identical to FTCD; formimino transferase
                    cyclodeaminase; formimino transferase (EC 2.1.2.5)
                    /formimino tetrahydro folate cyclodeaminase (EC 4.3.1.4)"
    """
    return {
        "type": "source",
        "location": {"type": "span", "from": 1, "to": 1326},
        "properties": {
            "organism": "Homo sapiens",
            "db_xref": ["taxon:9606"],
            "chromosome": "21",
            "clone": "IMAGE cDNA clone 125195",
            "clone_lib": "Soares fetal liver spleen 1NFLS",
            "note": t.normalize("""
                contains Alu repeat; likely to be be derived from
                unprocessed nuclear RNA or genomic DNA; encodes putative
                exons identical to FTCD; formimino transferase
                cyclodeaminase; formimino transferase (EC 2.1.2.5)
                /formimino tetrahydro folate cyclodeaminase (EC 4.3.1.4)
            """)
        }    
    }

@t.parse("parse_feature")
def test_weird_qualifier_name():
    """
    3'UTR           3341..3390
                    /gene="ITSN1"
    """
    return {
        "type": "3'UTR",
        "location": {"type": "span", "from": 3341, "to": 3390},
        "properties": {
            "gene": "ITSN1"
        }
    }

@t.parse("parse_feature")
def test_no_qualifiers():
    """
    sig_peptide     1..21
    """
    return {
        "type": "sig_peptide",
        "location": {"type": "span", "from": 1, "to": 21},
        "properties": {}
    }

@t.parse("parse_feature")
def test_transl_except():
    """
    CDS             3205759..3205994
                    /gene="CROCCL2"
                    /codon_start=1
                    /transl_except=(pos:complement(3205818..3205820),aa:OTHER)
    """
    return {
        "type": "CDS",
        "location": {"type": "span", "from": 3205759, "to": 3205994},
        "properties": {
            "gene": "CROCCL2",
            "codon_start": 1,
            "transl_except": [{
                "acid": "OTHER",
                "location": {
                    "type": "complement",
                    "segment": {
                        "type": "span",
                        "from": 3205818,
                        "to": 3205820,
                    }
                }
            }]
        }
    }


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "NP_001832"
    assert rec.locus["length"] == 360
    assert rec.locus["base_type"] == "aa"

def test_kw_length(rec):
    assert len(rec.keywords) == 9
    assert len(rec.keywords["reference"]) == 3
    assert len(rec.keywords["source"]["taxonomy"]) == 12

def test_feature_length(rec):
    assert len(rec.features) == 4
    
def test_feature_prop_lengths(rec):
    data = [4, 1, 3, 4]
    for r, e in zip(rec.features, data):
        assert len(r["properties"]) == e

def test_pseudo(rec):
    assert rec.features[3] == {
        "type": "CDS",
        "location": {"type": "span", "from": 1, "to": 360},
        "properties": {
            "pseudo": True,
            "gene": "CNR2",
            "db_xref": ["LocusID:1269", "MIM:605051"],
            "coded_by": "NM_001841.1:127..1209"
        }
    }

def test_sequence(rec):
    data = "".join("""
        meecwvteia ngskdgldsn pmkdymilsg pqktavavlc tllgllsale nvavlylils
        shqlrrkpsy lfigslagad flasvvfacs fvnfhvfhgv dskavfllki gsvtmtftas
        vgsllltaid rylclrypps ykalltrgra lvtlgimwvl salvsylplm gwtccprpcs
        elfplipndy llswllfiaf lfsgiiytyg hvlwkahqhv aslsghqdrq vpgmarmrld
        vrlaktlglv lavllicwfp vlalmahsla ttlsdqvkka fafcsmlcli nsmvnpviya
        lrsgeirssa hhclahwkkc vrglgseake eaprssvtet eadgkitpwp dsrdldlsdc
    """.split())
    seq = ''.join([s for s in rec.seqiter])
    assert seq == data



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "EU143250"

def test_stem_loop(rec):
    assert rec.features[6] == {
        "type": "stem_loop",
        "location": {"type": "span", "from": 3506, "to": 3595},
        "properties": {}
    }


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "HUGLUT1"
    
def test_counts(rec):
    assert rec.base_count == {"A": 206, "C": 172, "G": 195, "T": 168}



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import t

@t.parse("parse_features")
def test_multiple_features():
    """
     source          1..182
                     /organism="Mus musculus"
                     /db_xref="taxon:10090"
                     /chromosome="4"
                     /map="4 42.6 cM"
     Protein         1..182
                     /product="interferon beta, fibroblast"
     sig_peptide     1..21
     Region          1..182
                     /region_name="Interferon alpha/beta domain"
                     /db_xref="CDD:pfam00143"
                     /note="interferon"
     mat_peptide     22..182
                     /product="ifn-beta"
     Region          56..170
                     /region_name="Interferon alpha, beta and delta."
                     /db_xref="CDD:IFabd"
                     /note="IFabd"
     CDS             1..182
                     /gene="Ifnb"
                     /db_xref="LocusID:15977"
                     /db_xref="MGD:MGI:107657"
                     /coded_by="NM_010510.1:21..569"
    """
    return [
        {
            "type": "source",
            "location": {"type": "span", "from": 1, "to": 182},
            "properties": {
                "organism": "Mus musculus",
                "db_xref": ["taxon:10090"],
                "chromosome": "4",
                "map": "4 42.6 cM"
            }
        },
        {
            "type": "Protein",
            "location": {"type": "span", "from": 1, "to": 182},
            "properties": {"product": "interferon beta, fibroblast"}
        },
        {
            "type": "sig_peptide",
            "location": {"type": "span", "from": 1, "to": 21},
            "properties": {}
        },
        {
            "type": "Region",
            "location": {"type": "span", "from": 1, "to": 182},
            "properties": {
                "region_name": "Interferon alpha/beta domain",
                "db_xref": ["CDD:pfam00143"],
                "note": "interferon"
            }
        },
        {
            "type": "mat_peptide",
            "location": {"type": "span", "from": 22, "to": 182},
            "properties": {"product": "ifn-beta"}
        },
        {
            "type": "Region",
            "location": {"type": "span", "from": 56, "to": 170},
            "properties": {
                "region_name": "Interferon alpha, beta and delta.",
                "db_xref": ["CDD:IFabd"],
                "note": "IFabd"
            }
        },
        {
            "type": "CDS",
            "location": {"type": "span", "from": 1, "to": 182},
            "properties": {
                "gene": "Ifnb",
                "db_xref": ["LocusID:15977", "MGD:MGI:107657"],
                "coded_by": "NM_010510.1:21..569"
            }
        }
    ]



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "HSTMPO2"
    
def test_segment(rec):
    assert rec.keywords["segment"] == {"position": 2, "of": 6}


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import t

@t.parse("parse_base_count")
def test_1():
    "BASE COUNT    28300 a  15069 c  15360 g  27707 t"
    return {"A": 28300, "C": 15069, "G": 15360, "T": 27707}

@t.parse("parse_base_count")
def test_2():
    "BASE COUNT  1311257 a2224835 c2190093 g1309889 t"
    return {"A": 1311257, "C": 2224835, "G": 2190093, "T": 1309889}



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "NC_002678"
    
def test_counts(rec):
    assert rec.base_count == {
        "A": 1311257,
        "C": 2224835,
        "G": 2190093,
        "T": 1309889
    }
    



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "IRO125195"

def test_kw_length(rec):
    assert len(rec.keywords) == 7
    assert len(rec.keywords["reference"]) == 2

def test_feature_length(rec):
    assert len(rec.features) == 5
    
def test_feature_prop_lengths(rec):
    data = [6, 1, 2, 2, 2]
    for r, e in zip(rec.features, data):
        assert len(r["properties"]) == e

def test_first_bad_qual(rec):
    data = ' '.join("""
          contains Alu repeat; likely to be be derived from
          unprocessed nuclear RNA or genomic DNA; encodes putative
          exons identical to FTCD; formimino transferase
          cyclodeaminase; formimino transferase (EC 2.1.2.5)
          /formimino tetrahydro folate cyclodeaminase (EC 4.3.1.4)
    """.split())
    assert data == rec.features[0]["properties"]["note"]

def test_sequence(rec):
    data = "".join("""
        cacaggccca gagccactcc tgcctacagg ttctgagggc tcaggggacc tcctgggccc
        tcaggctctt tagctgagaa taagggccct gagggaacta cctgcttctc acatccccgg
        gtctctgacc atctgctgtg tgccccgacc ccccctaccc tgctcctcca ccaagcctga
        tgccaagggc tataaaccac tggcccaaca gaagcttggt tcccagagaa ctggtccctg
        cctgggacat gctccttgct acagcccctt gtgggagctc agagggcatg gctgctcccc
        ctacggtccc tcgcccagtg gttctgtctc tttatggcag gaagcaatga ggctccccaa
        gaacacacct gaggaaaagg acaggtgagc ctggagggcc ggccgcaccg tgggcctctg
        tgtctgggga gttggtggcc aggatcccga gtacctgggt gctgtgacgg ggcgtggttg
        gcctgggcgt gctgggtgtt tgggaatgac ttcccatcgc ccgcttctgc agcctgctca
        gccctgttgg ggtgcagtgt gtccactgcc tgcctgtgtg tgccgctgtg ctcaggctct
        cctcttgctc ctttcaggcg cacggcggcc ctacaggagg gtctgaggcg ggcagtctct
        gtgccgctga cgctggcgga gacggtggcc tcgctgtggc cggccctgca ggaactggcc
        cggtgtggga acctggcctg ccggtcagac ctccaggtag gggggcccgg gggaccccag
        gcctcctgcc gcaaagcagg aagcagctgt tggggctgag ttgctggaga gcacggtggt
        tggctgggct gagccaggta gggagacctc acttcacggg cagttcccgg ggctttggcc
        tcctccaaca gggccggggt gtccgctgcc ttctaagatc tcgctcacgg cggcaccaca
        gacggagacc caaatgtgtc tgcagagccc accctgacca ctgtataagt gtatactccc
        ccaagacccc ttgtatcacc cccactgtcg tgttctagaa aacacctatg tcagcccagg
        cacaatggtt ctcgcctgtg gtcccagcac tttgggaggc tgaggcggga ggatcacttg
        aatagaggag gtcgagacca gcctgggcaa catagtggga ccctggctgt acaatagata
        catatgagcc aggcacggtg gtgcctgtgg tcccagctcc ctgcccagcc agcctcctgt
        ctccagagaa gttctccatt aaaaaataat ttagcaaaaa aaaaaaaaaa aaaaaaaaaa
        aaaaaa
    """.split())
    seq = ''.join([s for s in rec.seqiter])
    assert seq == data


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "EU032008"

def test_kw_length(rec):
    assert len(rec.keywords) == 6
    assert len(rec.keywords["reference"]) == 2

def test_feature_length(rec):
    assert len(rec.features) == 3
    
def test_feature_prop_lengths(rec):
    data = [8, 1, 7]
    for r, e in zip(rec.features, data):
        assert len(r["properties"]) == e

def test_first_bad_qual(rec):
    assert rec.features[0]["properties"]["virion"] == True



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "ATKIN2"

def test_keyword(rec):
    assert rec.keywords["source"]["name"] == "thale cress"

def test_feature(rec):
    assert rec.features[2]["properties"]["number"] == 1

def test_counts(rec):
    assert rec.base_count == {"A": 263, "C": 155, "G": 160, "T": 302}

def test_sequence(rec):
    data = """
        atttggccta taaatataaa cccttaagcc cacatatctt ctcaatccat cacaaacaaa
        acacacatca aaaacgattt tacaagaaaa aaatatctga aaaatgtcag agaccaacaa
        gaatgccttc caagccggtc aggccgctgg caaagctgag gtactctttc tctcttagaa
        cagagtactg atagattgtt caagttataa ctctttgaaa acagttgaaa cttgatcact
        cctagaactt ccattttctt gtttaattta gtttgtcgta attatgtaat tgattttgtg
        ttgaccatgg ttgttatata ggagaagagc aatgttctgc tggacaaggc caaggatgct
        gctgctgcag ctggagcttc cgcgcaacag gtaaacgatc tatacacaca ttatgacatt
        tatgtaaaga atgaaaagtc ttcttagagc atacatttac gcagatttct gatattttca
        tatggtttga tgtaaatgtt ataggcggga aagagtatat cggatgcggc agtgggaggt
        gttaacttcg tgaaggacaa gaccggcctg aacaagtagc gatccgagtc aactttggga
        gttataattt cccttttcta attaattgtt gggattttca aataaaattt gggagtcata
        attgattctc gtactcatcg tacttgttgt tgtttttagt gttgtaatgt tttaatgttt
        cttctccctt tagatgtact acgtttggaa ctttaagttt aatcaacaaa atctagttta
        agttctaaga actttgtttt accatcctct tttttattgc acttaatgct tatagacttt
        tatctccatc catttctcaa ttcggctacg ttgaattata
    """
    assert ''.join(s for s in rec.seqiter) == ''.join(data.split())


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "EU071348"

def test_feature_length(rec):
    assert len(rec.features) == 3

def test_fuzzy_from(rec):
    assert rec.features[1]["location"] == {
        "type": "span",
        "from": 1,
        "to": 51,
        "modifiers": {"from": "fuzzy"}
    }

def test_fuzzy_to(rec):
    assert rec.features[2]["location"] == {
        "type": "span",
        "from": 52,
        "to": 705,
        "modifiers": {"to": "fuzzy"}
    }



def test_name(rec):
    assert rec.locus["name"] == "NC_008512"

def test_reference(rec):
    assert rec.keywords["reference"][0] == {
        'title': 'The 160-kilobase genome of the bacterial endosymbiont Carsonella',
        'journal': 'Science 314 (5797), 267 (2006)',
        'refid': 1,
        'pubmed': '17038615',
        'location': None,
        'notes': ''
    }



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "AF297471"

def test_keyword(rec):
    assert rec.keywords["accession"] == {"primary": "AF297471", "secondary": []}

def test_feature(rec):
    assert rec.features[0]["properties"]["cultivar"] == "Cascade"

def test_counts(rec):
    assert rec.base_count == {"A": 155, "C": 89, "G": 116, "T": 137}

def test_sequence(rec):
    data = """
        atggcagaca acaagcagag cttccaagcc ggtcaagccg ctggtcgtgc tgaggtctct
        ttctctcttt aactctctga tcaaatgcag ttgctgcggt taaacaatca cactttataa
        aataaagaga agttatttat attttcttga accaagtaaa tgaatgtatt ttgaccaaaa
        aaagaagaag taaatgaatg tattaattga tttcgtgtgg acaatggtgt gtaaatatag
        gagaagggta atgtgctgat ggacaaggtc aaggatgctg ctaccgcagc tggagcgtct
        gcgcaaaccg taagcgatct accccatgat atataaataa acatacgtct ctactctcta
        ctgtacatgt tcgtagatct catatctttt tatgctgttc taacatgtat ttacgtttat
        aggcgggaca gaagataacg gaggcggcag ggggagccgt taatctcgtg aaggagaaga
        ccggcatgaa caagtag
    """
    assert ''.join(s for s in rec.seqiter) == ''.join(data.split())



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "NM_015658"

def test_primary(rec):
    assert rec.keywords["primary"] == [
        {
            "refseq_span": {"from": 1, "to": 27},
            "primary_identifier": "DA002961.1",
            "primary_span": {"from": 1, "to": 27},
            "complement": False
        },
        {
            "refseq_span": {"from": 28, "to": 976},
            "primary_identifier": "BC003555.1",
            "primary_span": {"from": 1, "to": 949},
            "complement": False
        },
        {
            "refseq_span": {"from": 977, "to": 984},
            "primary_identifier": "AW749585.1",
            "primary_span": {"from": 25, "to": 32},
            "complement": False
        },
        {
            "refseq_span": {"from": 985, "to": 1901},
            "primary_identifier": "BC003555.1",
            "primary_span": {"from": 958, "to": 1874},
            "complement": False
        },
        {
            "refseq_span": {"from": 1902, "to": 2301},
            "primary_identifier": "AK225239.1",
            "primary_span": {"from": 1857, "to": 2256},
            "complement": False
        },
        {
            "refseq_span": {"from": 2302, "to": 2695},
            "primary_identifier": "BC003555.1",
            "primary_span": {"from": 2275, "to": 2668},
            "complement": False
        },
        {
            "refseq_span": {"from": 2696, "to": 2789},
            "primary_identifier": "BX645732.1",
            "primary_span": {"from": 174, "to": 267},
            "complement": False
        },
        {
            "refseq_span": {"from": 2790, "to": 2817},
            "primary_identifier": "AK225239.1",
            "primary_span": {"from": 2739, "to": 2766},
            "complement": False
        }
    ]



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import datetime

def test_locus(rec):
    assert rec.locus == {
        "name": "EU140800",
        "length": 3390,
        "base_type": "bp",
        "molecule_type": "mRNA",
        "topology": "linear",
        "division": "PRI",
        "date": datetime.date(2007, 9, 29)
    }

def test_keywords(rec):
    assert rec.keywords == {
        "definition": " ".join("""
            Homo sapiens intersectin 1 short form variant 3 (ITSN1) mRNA,
            complete cds, alternatively spliced
        """.split()),
        "accession": {"primary": "EU140800", "secondary": []},
        "version": {"accession": "EU140800.1", "gi": "157497185"},
        "keywords": [],
        "source": {
            "name": "Homo sapiens (human)",
            "organism": "Homo sapiens",
            "taxonomy": """
                Eukaryota Metazoa Chordata Craniata Vertebrata Euteleostomi
                Mammalia Eutheria Euarchontoglires Primates Haplorrhini
                Catarrhini Hominidae Homo
            """.split()
        },
        "reference": [{
            "refid": 1,
            "location": {"from": 1, "to": 3390},
            "notes": "",
            "authors": "Kropyvko,S., Tsyba,L., Skrypkina,I. and Rynditch,A.",
            "title": " ".join("""
                Isolation and characterization of the novel complete coding
                sequence isoforms of intersectin 1
            """.split()),
            "journal": "Unpublished"
        },{
            "refid": 2,
            "location": {"from": 1, "to": 3390},
            "notes": "",
            "authors": "Kropyvko,S., Tsyba,L., Skrypkina,I. and Rynditch,A.",
            "title": "Direct Submission",
            "journal": " ".join("""
                Submitted (06-SEP-2007) Department of Functional Genomics,
                Institute of Molecular Biology and Genetics, 150 Zabolotnogo Str.,
                Kyiv 03143, Ukraine
            """.split())
        }]
    }

def test_features(rec):
    assert rec.features == [
        {
            "type": "source",
            "location": {"type": "span", "from": 1, "to": 3390},
            "properties": {
                "organism": "Homo sapiens",
                "mol_type": "mRNA",
                "db_xref": ["taxon:9606"],
                "chromosome": "21",
                "map": "21q22.1-q22.2",
                "tissue_type": "brain",
                "dev_stage": "12 week embryo"
            }
        },
        {
            "type": "gene",
            "location": {"type": "span", "from": 1, "to": 3390},
            "properties": {
                "gene": "ITSN1"
            }
        },
        {
            "type": "CDS",
            "location": {"type": "span", "from": 2, "to": 3340},
            "properties": {
                "gene": "ITSN1",
                "note": " ".join("""
                    alternatively spliced; produced by truncation of
                    exon 6 and alternative splicing of exons 25 and 26
                """.split()),
                "codon_start": 1,
                "product": "intersectin 1 short form variant 3",
                "protein_id": "ABV58336.1",
                "db_xref": ["GI:157497186"],
                "translation": "".join("""
                    MAQFPTPFGGSLDIWAITVEERAKHDQQFHSLKPISGFITGDQA
                    RNFFFQSGLPQPVLAQIWALADMNNDGRMDQVEFSIAMKLIKLKLQGYQLPSALPPVM
                    KQQPVAISSAPAFAAVPPLANGAPPVIQPLPAFAHPAATLPKSSSFSRSGPGSQLNTK
                    LQKAQSFDVASVPPVAEWAVPQSSRLKYRQLFNSHDKTMSGHLTGPQARTILMQSSLP
                    QAQLASIWNLSDIDQDGKLTAEEFILAMHLIDVAMSGQPLPPVLPPEYIPPSFRRVRS
                    GSGISVISSTSVDQRLPEEPVLEDEQQQLEKKLPVTFEDKKRENFERGNLELEKRRQA
                    LLEQQRKEQERLAQLERAEQERKERERQEQERKRQLELEKQLEKQRELERQREEERRK
                    EIERREAAKRELERQRQLEWERNRRQELLNQRNKEQEDIVVLKAKKKTLEFELEALND
                    KKHQLEGKLQDIRCRLTTQRQEIESTNKSRELRIAEITHLQQQLQESQQMLGRLIPEK
                    QILNDQLKQVQQNSLHRDSLVTLKRALEAKELARQHLRDQLDEVEKETRSKLQEIDIF
                    NNQLKELREIHNKQQLQKQKSMEAERLKQKEQERKIIELEKQKEEAQRRAQERDKQWL
                    EHVQQEDEHQRPRKLHEEEKLKREESVKKKDGEEKGKQEAQDKLGRLFHQHQEPAKPA
                    VQAPWSTAEKGPLTISAQENVKVVYYRALYPFESRSHDEITIQPGDIVMVKGEWVDES
                    QTGEPGWLGGELKGKTGWFPANYAEKIPENEVPAPVKPVTDSTSAPAPKLALRETPAP
                    LAVTSSEPSTTPNNWADFSSTWPTSTNEKPETDNWDAWAAQPSLTVPSAGQLRQRSAF
                    TPATATGSSPSPVLGQGEKVEGLQAQALYPWRAKKDNHLNFNKNDVITVLEQQDMWWF
                    GEVQGQKGWFPKSYVKLISGPIRKSTSMDSGSSESPASLKRVASPAAKPVVSGEEIAQ
                    VIASYTATGPEQLTLAPGQLILIRKKNPGGWWEGELQARGKKRQIGWFPANYVKLLSP
                    GTSKITPTEPPKSTALAAVCQVIGMYDYTAQNDDELAFNKGQIINVLNKEDPDWWKGE
                    VNGQVGLFPSNYVKLTTDMDPSQQ
                """.split())
            }
        },
        {
            "type": "3'UTR",
            "location": {"type": "span", "from": 3341, "to": 3390},
            "properties": {
                "gene": "ITSN1"
            }
        }
    ]

def test_sequence(rec):
    data = """
        catggctcagtttccaacaccttttggtggcagcctggatatctgggccataactgtaga
        ggaaagagcgaagcatgatcagcagttccatagtttaaagccaatatctggattcattac
        tggtgatcaagctagaaactttttttttcaatctgggttacctcaacctgttttagcaca
        gatatgggcactagctgacatgaataatgatggaagaatggatcaagtggagttttccat
        agctatgaaacttatcaaactgaagctacaaggatatcagctaccctctgcacttccccc
        tgtcatgaaacagcaaccagttgctatttctagcgcaccagcatttgcagctgtgccccc
        cctggctaacggggctccccctgttatacaacctctgcctgcatttgctcatcctgcagc
        cacattgccaaagagttcttcctttagtagatctggtccagggtcacaactaaacactaa
        attacaaaaggcacagtcatttgatgtggccagtgtcccaccagtggcagagtgggctgt
        tcctcagtcatcaagactgaaatacaggcaattattcaatagtcatgacaaaactatgag
        tggacacttaacaggtccccaagcaagaactattcttatgcagtcaagtttaccacaggc
        tcagctggcttcaatatggaatctttctgacattgatcaagatggaaaacttacagcaga
        ggaatttatcctggcaatgcacctcattgatgtagctatgtctggccaaccactgccacc
        tgtcctgcctccagaatacattccaccttcttttagaagagttcgatctggcagtggtat
        atctgtcataagctcaacatctgtagatcagaggctaccagaggaaccagttttagaaga
        tgaacaacaacaattagaaaagaaattacctgtaacgtttgaagataagaagcgggagaa
        ctttgaacgtggcaacctggaactggagaaacgaaggcaagctctcctggaacagcagcg
        caaggagcaggagcgcctggcccagctggagcgggcggagcaggagaggaaggagcgtga
        gcgccaggagcaagagcgcaaaagacaactggaactggagaagcaactggaaaagcagcg
        ggagctagaacggcagagagaggaggagaggaggaaagaaattgagaggcgagaggctgc
        aaaacgggaacttgaaaggcaacgacaacttgagtgggaacggaatcgaaggcaagaact
        actaaatcaaagaaacaaagaacaagaggacatagttgtactgaaagcaaagaaaaagac
        tttggaatttgaattagaagctctaaatgataaaaagcatcaactagaagggaaacttca
        agatatcagatgtcgattgaccacccaaaggcaagaaattgagagcacaaacaaatctag
        agagttgagaattgccgaaatcacccatctacagcaacaattacaggaatctcagcaaat
        gcttggaagacttattccagaaaaacagatactcaatgaccaattaaaacaagttcagca
        gaacagtttgcacagagattcacttgttacacttaaaagagccttagaagcaaaagaact
        agctcggcagcacctacgagaccaactggatgaagtggagaaagaaactagatcaaaact
        acaggagattgatattttcaataatcagctgaaggaactaagagaaatacacaataagca
        acaactccagaagcaaaagtccatggaggctgaacgactgaaacagaaagaacaagaacg
        aaagatcatagaattagaaaaacaaaaagaagaagcccaaagacgagctcaggaaaggga
        caagcagtggctggagcatgtgcagcaggaggacgagcatcagagaccaagaaaactcca
        cgaagaggaaaaactgaaaagggaggagagtgtcaaaaagaaggatggcgaggaaaaagg
        caaacaggaagcacaagacaagctgggtcggcttttccatcaacaccaagaaccagctaa
        gccagctgtccaggcaccctggtccactgcagaaaaaggtccacttaccatttctgcaca
        ggaaaatgtaaaagtggtgtattaccgggcactgtacccctttgaatccagaagccatga
        tgaaatcactatccagccaggagacatagtcatggttaaaggggaatgggtggatgaaag
        ccaaactggagaacccggctggcttggaggagaattaaaaggaaagacagggtggttccc
        tgcaaactatgcagagaaaatcccagaaaatgaggttcccgctccagtgaaaccagtgac
        tgattcaacatctgcccctgcccccaaactggccttgcgtgagacccccgcccctttggc
        agtaacctcttcagagccctccacgacccctaataactgggccgacttcagctccacgtg
        gcccaccagcacgaatgagaaaccagaaacggataactgggatgcatgggcagcccagcc
        ctctctcaccgttccaagtgccggccagttaaggcagaggtccgcctttactccagccac
        ggccactggctcctccccgtctcctgtgctaggccagggtgaaaaggtggaggggctaca
        agctcaagccctatatccttggagagccaaaaaagacaaccacttaaattttaacaaaaa
        tgatgtcatcaccgtcctggaacagcaagacatgtggtggtttggagaagttcaaggtca
        gaagggttggttccccaagtcttacgtgaaactcatttcagggcccataaggaagtctac
        aagcatggattctggttcttcagagagtcctgctagtctaaagcgagtagcctctccagc
        agccaagccggtcgtttcgggagaagaaattgcccaggttattgcctcatacaccgccac
        cggccccgagcagctcactctcgcccctggtcagctgattttgatccgaaaaaagaaccc
        aggtggatggtgggaaggagagctgcaagcacgtgggaaaaagcgccagataggctggtt
        cccagctaattatgtaaagcttctaagccctgggacgagcaaaatcactccaacagagcc
        acctaagtcaacagcattagcggcagtgtgccaggtgattgggatgtacgactacaccgc
        gcagaatgacgatgagctggccttcaacaagggccagatcatcaacgtcctcaacaagga
        ggaccctgactggtggaaaggagaagtcaatggacaagtggggctcttcccatccaatta
        tgtgaagctgaccacagacatggacccaagccagcaatgaatcatatgttgtccatcccc
        ccctcaggcttgaaagtcctcaaagagacc
    """
    assert ''.join(s for s in rec.seqiter) == ''.join(data.split())



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "HSTMPO1"
    
def test_one_of(rec):
    assert rec.features[3]["location"] == {
        "type": "span",
        "from": {
            "type": "one-of",
            "positions": [1888, 1901]
        },
        "to": 2479
    }


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import datetime
import itertools
import pprint
import textwrap
import types

import nebgb

def fmt(path, msg, rec, exp):
    return "PATH: %s\n%s\nFound:\n%s\nExpected:\n%s" % (
        path,
        msg,
        pprint.pformat(rec),
        pprint.pformat(exp)
    )

def mkpath(base, next):
    if base is None:
        return next
    return "%s.%s" % (base, next)

def sameobject(rec, exp, path=None):
    assert rec.__class__ == exp.__class__, fmt(path, "Type mismatch", rec, exp)
    func = DISPATCH[rec.__class__](rec, exp, path)

def isempty(it):
    try:
        it.next()
    except StopIteration:
        return True
    return False

def samesequence(rec, exp):
    for idx, (r, e) in enumerate(itertools.izip(rec, exp)):
        sameobject(r, e, path="%d" % idx)
    assert isempty(rec), "Record iterator is not empty."
    assert isempty(exp), "Expect iterator is not empty."        

def bothnone(rec, exp, path=None):
    assert rec is None and exp is None, fmt(path, "None mistmatch", rec, exp)

def samebool(rec, exp, path=None):
    assert rec == exp, fmt(path, "Bool mismatch", rec, exp)

def sameint(rec, exp, path=None):
    assert rec == exp, fmt(path, "Int mismatch", rec, exp)

def samestr(rec, exp, path=None):
    assert rec == exp, fmt(path, "Str mismatch", rec, exp)

def samelist(rec, exp, path=None):
    assert len(rec) == len(exp), fmt(path, "Length mismatch", rec, exp)
    for idx, (r, e) in enumerate(zip(rec, exp)):
        sameobject(r, e, mkpath(path, "%d" % idx))

def samedict(rec, exp, path=None):
    rkeys, ekeys = list(rec.keys()), list(exp.keys())
    rkeys.sort(), ekeys.sort()
    samelist(rkeys, ekeys, mkpath(path, "__KEYS__"))
    for key in rkeys:
        sameobject(rec[key], exp[key], mkpath(path, key))

def samedate(rec, exp, path=None):
    assert rec == exp, fmt(path, "Date mismatch", rec, exp)

DISPATCH = {
    types.NoneType: bothnone,
    bool: samebool,
    int: sameint,
    str: samestr,
    list: samelist,
    tuple: samelist, 
    dict: samedict,
    datetime.date: samedate
}


class parse(object):
    def __init__(self, name):
        self.name = name
    def __call__(self, func):
        def wrap(*args, **kwargs):
            data = self.prep(func.func_doc)
            result = getattr(nebgb, self.name)(data)
            sameobject(result, func())
        wrap.func_name = func.func_name
        return wrap
    def prep(self, data):
        return data.lstrip("\n").rstrip(" ")


def normalize(data):
    return ' '.join(data.split())

def dedent(data, left=None, right=None):
    return textwrap.dedent(data.lstrip(left).rstrip(right))


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import t

@t.parse("parse_keyword")
def test_definition():
    """
DEFINITION  Pseudomonas syringae pv. averrhoi strain HL1 leucyl-tRNA synthetase
            (leuS) gene, partial cds; putative lipoprotein, DNA polymerase III
            delta subunit (holA), type III effector HopAJ2 (hopAJ2), lipoic
            acid synthetase (lipA), and lipoate-protein ligase B (lipB) genes,
            complete cds; and hypothetical protein gene, partial cds.
    """
    return t.normalize("""
        Pseudomonas syringae pv. averrhoi strain HL1 leucyl-tRNA synthetase
        (leuS) gene, partial cds; putative lipoprotein, DNA polymerase III
        delta subunit (holA), type III effector HopAJ2 (hopAJ2), lipoic
        acid synthetase (lipA), and lipoate-protein ligase B (lipB) genes,
        complete cds; and hypothetical protein gene, partial cds
    """)
    
@t.parse("parse_keyword")
def test_accession_1():
    "ACCESSION   AL109817"
    return {"primary": "AL109817", "secondary": []}

@t.parse("parse_keyword")
def test_accession_2():
    "ACCESSION   AL109817   NC01231-NC01250"
    return {
        "primary": "AL109817",
        "secondary": [{"first": "NC01231", "last": "NC01250"}]
    }
    
@t.parse("parse_keyword")
def test_accession_3():
    "ACCESSION AL109817 NC01231-NC01250 AL109818"
    return {
        "primary": "AL109817",
        "secondary": [
            {"first": "NC01231", "last": "NC01250"},
            "AL109818"
        ]
    }

@t.parse("parse_keyword")
def test_version():
    "VERSION NC_0012312.4 GI:23923234"
    return {"accession": "NC_0012312.4", "gi": "23923234"}

@t.parse("parse_keyword")
def test_pid():
    "PID         g134354"
    return "g134354"

@t.parse("parse_keyword")
def test_dbsource():
    """
    DBSOURCE    swissprot: locus SCX3_BUTOC, accession P01485;
                class: standard.
                created: Jul 21, 1986.
                sequence updated: Jul 21, 1986.
                annotation updated: Oct 16, 2001.
                xrefs: gi: gi: 69530
    """
    return t.dedent("""
        swissprot: locus SCX3_BUTOC, accession P01485;
        class: standard.
        created: Jul 21, 1986.
        sequence updated: Jul 21, 1986.
        annotation updated: Oct 16, 2001.
        xrefs: gi: gi: 69530
    """, left="\n", right=None)

@t.parse("parse_keyword")
def test_segment():
    "SEGMENT     2 of 6"
    return {"position": 2, "of": 6}
    
@t.parse("parse_keyword")
def test_keywords_1():
    "KEYWORDS    ."
    return []

@t.parse("parse_keyword")
def test_keywords_2():
    "KEYWORDS    FLI_CDNA."
    return ["FLI_CDNA"]

@t.parse("parse_keyword")
def test_keywords_3():
    "KEYWORDS    Neurotoxin; Sodium channel inhibitor; Amidation."
    return ["Neurotoxin", "Sodium channel inhibitor", "Amidation"]

@t.parse("parse_keyword")
def test_keywords_4():
    """
KEYWORDS    antifreeze protein homology; cold-regulated gene; cor6.6 gene; KIN1
            homology.
    """
    return [
        "antifreeze protein homology",
        "cold-regulated gene",
        "cor6.6 gene",
        "KIN1 homology"
    ]

@t.parse("parse_keyword")
def test_source():
    """
    SOURCE      Human immunodeficiency virus 1 (HIV-1)
      ORGANISM  Human immunodeficiency virus 1
                Viruses; Retro-transcribing viruses; Retroviridae;
                Orthoretrovirinae; Lentivirus; Primate lentivirus group.
    """
    return {
        "name": "Human immunodeficiency virus 1 (HIV-1)",
        "organism": "Human immunodeficiency virus 1",
        "taxonomy": [
            "Viruses",
            "Retro-transcribing viruses",
            "Retroviridae",
            "Orthoretrovirinae",
            "Lentivirus",
            "Primate lentivirus group"
        ]
    }

@t.parse("parse_keyword")
def test_reference_1():
    """
REFERENCE   2  (bases 1 to 1005)
  AUTHORS   Paraschiv,S., Tinischi,M., Neaga,E., Costache,M., DInu,M.,
            Florea,M. and Otelea,D.
  TITLE     Direct Submission
  JOURNAL   Submitted (13-JUL-2007) Molecular Diagnostics Laboratory, Institute
            for Infectious Diseases 'Prof. Dr. Matei Bals', Grozovici Street,
            No. 1, Sector 2, Bucharest 021105, Romania
    """
    return {
        "refid": 2,
        "location": {"from": 1, "to": 1005},
        "notes": "",
        "authors": t.normalize("""
            Paraschiv,S., Tinischi,M., Neaga,E., Costache,M., DInu,M.,
            Florea,M. and Otelea,D.
        """),
        "title": "Direct Submission",
        "journal": t.normalize("""
            Submitted (13-JUL-2007) Molecular Diagnostics Laboratory, Institute
            for Infectious Diseases 'Prof. Dr. Matei Bals', Grozovici Street,
            No. 1, Sector 2, Bucharest 021105, Romania
        """)
    }
    
@t.parse("parse_keyword")
def test_reference_2():
    """
REFERENCE   1  (sites)
  AUTHORS   Kaneko,T., Nakamura,Y., Sato,S., Asamizu,E., Kato,T., Sasamoto,S.,
            Watanabe,A., Idesawa,K., Ishikawa,A., Kawashima,K., Kimura,T.,
            Kishida,Y., Kiyokawa,C., Kohara,M., Matsumoto,M., Matsuno,A.,
            Mochizuki,Y., Nakayama,S., Nakazaki,N., Shimpo,S., Sugimoto,M.,
            Takeuchi,C., Yamada,M. and Tabata,S.
  TITLE     Complete genome structure of the nitrogen-fixing symbiotic
            bacterium Mesorhizobium loti
  JOURNAL   DNA Res. 7, 331-338 (2000)
    """
    return {
        "refid": 1,
        "location": None,
        "notes": "sites",
        "authors": t.normalize("""
            Kaneko,T., Nakamura,Y., Sato,S., Asamizu,E., Kato,T., Sasamoto,S.,
            Watanabe,A., Idesawa,K., Ishikawa,A., Kawashima,K., Kimura,T.,
            Kishida,Y., Kiyokawa,C., Kohara,M., Matsumoto,M., Matsuno,A.,
            Mochizuki,Y., Nakayama,S., Nakazaki,N., Shimpo,S., Sugimoto,M.,
            Takeuchi,C., Yamada,M. and Tabata,S.
        """),
        "title": t.normalize("""
            Complete genome structure of the nitrogen-fixing symbiotic
            bacterium Mesorhizobium loti
        """),
        "journal": "DNA Res. 7, 331-338 (2000)"
    }

@t.parse("parse_keyword")
def test_primary_1():
    """
    PRIMARY     REFSEQ_SPAN         PRIMARY_IDENTIFIER PRIMARY_SPAN        COMP
                1-27                DA002961.1         1-27
                28-976              BC003555.1         1-949
                977-984             AW749585.1         25-32
    """
    return [
        {
            "refseq_span": {"from": 1, "to": 27},
            "primary_identifier": "DA002961.1",
            "primary_span": {"from": 1, "to": 27},
            "complement": False
        },
        {
            "refseq_span": {"from": 28, "to": 976},
            "primary_identifier": "BC003555.1",
            "primary_span": {"from": 1, "to": 949},
            "complement": False,
        },
        {
            "refseq_span": {"from": 977, "to": 984},
            "primary_identifier": "AW749585.1",
            "primary_span": {"from": 25, "to": 32},
            "complement": False
        }
    ]

@t.parse("parse_keyword")
def test_primary_2():
    """
    PRIMARY     REFSEQ_SPAN         PRIMARY_IDENTIFIER PRIMARY_SPAN        COMP
                1-635               CF172065.1         1-635
                636-2166            AC138613.6         85778-87308         c
                2167-2424           AI839133.1         1-258               c
    """
    return [
        {
            "refseq_span": {"from": 1, "to": 635},
            "primary_identifier": "CF172065.1",
            "primary_span": {"from": 1, "to": 635},
            "complement": False
        },
        {
            "refseq_span": {"from": 636, "to": 2166},
            "primary_identifier": "AC138613.6",
            "primary_span": {"from": 85778, "to": 87308},
            "complement": True,
        },
        {
            "refseq_span": {"from": 2167, "to": 2424},
            "primary_identifier": "AI839133.1",
            "primary_span": {"from": 1, "to": 258},
            "complement": True
        }
    ]

@t.parse("parse_keyword")
def test_comment():
    """
COMMENT     VALIDATED REFSEQ: This record has undergone preliminary review of
            the sequence, but has not yet been subject to final review. The
            reference sequence was derived from DA002961.1, BC003555.1,
            AW749585.1, AK225239.1 and BX645732.1.
            On Sep 27, 2007 this sequence version replaced gi:142373530.
            COMPLETENESS: complete on the 3' end.
    """
    return t.dedent("""
        VALIDATED REFSEQ: This record has undergone preliminary review of
        the sequence, but has not yet been subject to final review. The
        reference sequence was derived from DA002961.1, BC003555.1,
        AW749585.1, AK225239.1 and BX645732.1.
        On Sep 27, 2007 this sequence version replaced gi:142373530.
        COMPLETENESS: complete on the 3' end.
    """, left="\n")


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import datetime
import textwrap

def test_locus(rec):
    assert rec.locus == {
        "name": "NP_034640",
        "length": 182,
        "base_type": "aa",
        "division": "ROD",
        "date": datetime.date(2000, 11, 1)
    }

def test_keywords(rec):
    assert rec.keywords == {
        "definition": "interferon beta, fibroblast [Mus musculus]",
        "accession": {"primary": "NP_034640", "secondary": []},
        "pid": "g6754304",
        "version": {"accession": "NP_034640.1", "gi": "6754304"},
        "dbsource": "REFSEQ: accession NM_010510.1",
        "keywords": [],
        "source": {
            "name": "house mouse",
            "organism": "Mus musculus",
            "taxonomy": """
                Eukaryota Metazoa Chordata Craniata Vertebrata Euteleostomi
                Mammalia Eutheria Rodentia Sciurognathi Muridae Murinae Mus
            """.split()
        },
        "reference": [{
            "refid": 1,
            "location": {"from": 1, "to": 182},
            "notes": "",
            "authors": " ".join("""
                Higashi,Y., Sokawa,Y., Watanabe,Y., Kawade,Y., Ohno,S.,
                Takaoka,C. and Taniguchi,T.
            """.split()),
            "title": "structure and expression of a cloned cdna for mouse interferon-beta",
            "journal": "J. Biol. Chem. 258, 9522-9529 (1983)",
            "medline": "83265757",
        }],
        "comment": textwrap.dedent("""
            PROVISIONAL REFSEQ: This record has not yet been subject to final
            NCBI review. The reference sequence was derived from K00020.1.
        """.lstrip("\n").rstrip())
    }

def test_features(rec):
    assert rec.features == [
        {
            "type": "source",
            "location": {"type": "span", "from": 1, "to": 182},
            "properties": {
                "organism": "Mus musculus",
                "db_xref": ["taxon:10090"],
                "chromosome": "4",
                "map": "4 42.6 cM"
            }
        },
        {
            "type": "Protein",
            "location": {"type": "span", "from": 1, "to": 182},
            "properties": {
                "product": "interferon beta, fibroblast"
            }
        },
        {
            "type": "sig_peptide",
            "location": {"type": "span", "from": 1, "to": 21},
            "properties": {}
        },
        {
            "type": "Region",
            "location": {"type": "span", "from": 1, "to": 182},
            "properties": {
                "region_name": "Interferon alpha/beta domain",
                "db_xref": ["CDD:pfam00143"],
                "note": "interferon"
            }
        },
        {
            "type": "mat_peptide",
            "location": {"type": "span", "from": 22, "to": 182},
            "properties": {
                "product": "ifn-beta"
            }
        },
        {
            "type": "Region",
            "location": {"type": "span", "from": 56, "to": 170},
            "properties": {
                "region_name": "Interferon alpha, beta and delta.",
                "db_xref": ["CDD:IFabd"],
                "note": "IFabd"
            }
        },
        {
            "type": "CDS",
            "location": {"type": "span", "from": 1, "to": 182},
            "properties": {
                "gene": "Ifnb",
                "db_xref": ["LocusID:15977", "MGD:MGI:107657"],
                "coded_by": "NM_010510.1:21..569"
            }
        }
    ]

def test_sequence(rec):
    data = """
        mnnrwilhaa fllcfsttal sinykqlqlq ertnirkcqe lleqlngkin ltyradfkip
        memtekmqks ytafaiqeml qnvflvfrnn fsstgwneti vvrlldelhq qtvflktvle
        ekqeerltwe msstalhlks yywrvqrylk lmkynsyawm vvraeifrnf liirrltrnf
        qn
    """
    assert ''.join(s for s in rec.seqiter) == ''.join(data.split())



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "HSTMPO2"
    
def test_location(rec):
    assert rec.features[1]["location"] == {
        "type": "order",
        "segments": [
            {
                "type": "reference",
                "accession": "U18266.1",
                "segment": {"type": "span", "from": 1888, "to": 2509},
            },
            {"type": "span", "from": 1, "to": 270},
            {
                "type": "reference",
                "accession": "U18268.1",
                "segment": {"type": "span", "from": 1, "to": 309},
            },
            {
                "type": "reference",
                "accession": "U18270.1",
                "segment": {"type": "span", "from": 1, "to": 6905}
            },
            {
                "type": "reference",
                "accession": "U18269.1",
                "segment": {"type": "span", "from": 1, "to": 128}
            },
            {
                "type": "reference",
                "accession": "U18271.1",
                "segment": {"type": "span", "from": 1, "to": 3234}
            }
        ]
    }



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "DMBR25B3"

def test_kw_length(rec):
    assert len(rec.keywords) == 7
    assert len(rec.keywords["reference"]) == 2

def test_feature_length(rec):
    assert len(rec.features) == 25
    
def test_feature_prop_lengths(rec):
    data = [
        3,
        1, 6, 1, 6, 1, 6, 1, 6, 1, 6, 1, 6,
        1, 6, 1, 6, 1, 6, 1, 5, 1, 6, 1, 5
    ]
    for r, e in zip(rec.features, data):
        assert len(r["properties"]) == e

def test_first_bad_qual(rec):
    data = ' '.join("""
         /prediction=(method:''genefinder'',
         version:''084'', score:''105.71'');
         /prediction=(method:''genscan'', version:''1.0'');
         /match=(desc:''BASEMENT MEMBRANE-SPECIFIC HEPARAN SULFATE
         PROTEOGLYCAN CORE PROTEIN PRECURSOR (HSPG) (PERLECAN)
         (PLC)'', species:''Homo sapiens (Human)'',
         ranges:(query:24292..24549,
         target:SWISS-PROT::P98160:3713..3628, score:''201.00''),
         (query:24016..24291, target:SWISS-PROT::P98160:3815..3724,
         score:''139.00''), (query:23857..24006,
         target:SWISS-PROT::P98160:3866..3817, score:''99.00''),
         (query:24052..24327, target:SWISS-PROT::P98160:4059..3968,
         score:''143.00''), (query:24046..24312,
         target:SWISS-PROT::P98160:4341..4253, score:''116.00''),
         (query:23806..23901, target:SWISS-PROT::P98160:4177..4146,
         score:''76.00''), (query:23203..23382,
         target:SWISS-PROT::P98160:4062..4003, score:''116.00''),
         (query:22523..22777, target:SWISS-PROT::P98160:4288..4204,
         score:''112.00''), (query:22235..22300,
         target:SWISS-PROT::P98160:4358..4337, score:''64.00'')),
         method:''blastx'', version:''1.4.9'');
         /match=(desc:''GM03359.5prime GM Drosophila melanogaster
         ovary BlueScript Drosophila melanogaster cDNA clone
         GM03359 5prime, mRNA sequence'', species:''Drosophila
         melanogaster (fruit fly)'', ranges:(query:25024..25235,
         target:EMBL::AA801707:438..227, score:''1024.00''),
         (query:24851..24898, target:EMBL::AA801707:476..429,
         score:''204.00'')), method:''blastn'', version:''1.4.9'');
         /match=(desc:''LD08615.5prime LD Drosophila melanogaster
         embryo BlueScript Drosophila melanogaster cDNA clone
         LD08615 5prime, mRNA sequence'', species:''Drosophila
         melanogaster (fruit fly)'', ranges:(query:24629..24727,
         target:EMBL::AA264808:99..1, score:''495.00''),
         (query:24417..24566, target:EMBL::AA264808:250..101,
         score:''687.00''), (query:24048..24420,
         target:EMBL::AA264808:618..246, score:''1847.00''),
         (query:23986..24036, target:EMBL::AA264808:678..628,
         score:''237.00'')), method:''blastn'', version:''1.4.9'');
         /match=(desc:''HL02745.5prime HL Drosophila melanogaster
         head BlueScript Drosophila melanogaster cDNA clone HL02745
         5prime, mRNA sequence'', species:''Drosophila melanogaster
         (fruit fly)'', ranges:(query:23944..24045,
         target:EMBL::AA697546:103..2, score:''510.00''),
         (query:23630..23943, target:EMBL::AA697546:416..103,
         score:''1570.00''), (query:23419..23561,
         target:EMBL::AA697546:558..416, score:''715.00''),
         (query:23306..23417, target:EMBL::AA697546:670..559,
         score:''524.00''), (query:23280..23316,
         target:EMBL::AA697546:695..659, score:''167.00'')),
         method:''blastn'', version:''1.4.9'');
         /match=(desc:''GM08137.5prime GM Drosophila melanogaster
         ovary BlueScript Drosophila melanogaster cDNA clone
         GM08137 5prime, mRNA sequence'', species:''Drosophila
         melanogaster (fruit fly)'', ranges:(query:23235..23278,
         target:EMBL::AA696682:44..1, score:''139.00''),
         (query:22986..23251, target:EMBL::AA696682:294..29,
         score:''1321.00'')), method:''blastn'',
         version:''1.4.9'')
    """.split())
    assert data == rec.features[2]["properties"]["note"]

def test_base_counts(rec):
    assert rec.base_count == {"A": 42902, "C": 33790, "G": 34046, "T": 43591}


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

def test_name(rec):
    assert rec.locus["name"] == "NM_001003918"
    
def test_primary(rec):
    assert rec.keywords["primary"] == [
        {
            "refseq_span": {"from": 1, "to": 57},
            "primary_identifier": "AC156026.3",
            "primary_span": {"from": 220162, "to": 220218},
            "complement": False
        },
        {
            "refseq_span": {"from": 58, "to": 540},
            "primary_identifier": "AF548565.1",
            "primary_span": {"from": 1, "to": 483},
            "complement": False
        },
        {
            "refseq_span": {"from": 541, "to": 1137},
            "primary_identifier": "AK135814.1",
            "primary_span": {"from": 733, "to": 1329},
            "complement": False
        },
        {
            "refseq_span": {"from": 1138, "to": 3369},
            "primary_identifier": "AF548565.1",
            "primary_span": {"from": 1081, "to": 3312},
            "complement": False
        },
        {
            "refseq_span": {"from": 3370, "to": 5427},
            "primary_identifier": "AC115005.11",
            "primary_span": {"from": 42241, "to": 44298},
            "complement": False
        }
    ]



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import calendar
import datetime
import itertools
import re
import sys
import types
import warnings

from pyparsing import *

__all__ = ["Record", "Sequence", "parse", "parse_file"]

#
# Constants
#

STRANDS = "ss ds ms".split()
MOL_TYPES = """NS DNA RNA tRNA rRNA mRNA uRNA snRNA snoRNA""".split()
TOPOLOGY = "linear circular".split()
DIVISIONS = """
    PRI ROD MAM VRT INV PLN BCT VRL PHG SYN UNA EST PAT STS GSS HTG HTC ENV CON
""".split()
MONTHS = dict(
    (m.upper(), i) for i, m in enumerate(calendar.month_abbr) if m
)

MULTI_KEYWORDS = set(["reference"])
MULTI_QUALIFIERS = set("""
    allele
    db_xref
    ec_number
    function
    go_component go_process go_function
    mobile_element
    old_locus_tag
    transl_except
""".split())
#
# Parse Actions
#

def to_int(t):
    return int(t[0])

def to_month(s, l, t):
    global MONTHS
    if t[0].upper() not in MONTHS:
        raise ParseException(s, len(s), "Expected a month abbreviation.")
    return MONTHS[t[0].upper()]

def to_date(s, l, t):
    day = t[0][0]
    month = t[0][1]
    year = t[0][2]
    return datetime.date(year, month, day)

#
# Grammar Definition
#

# General
integer = Word(nums).setParseAction(to_int)
normalized = Combine(
    ZeroOrMore(Word(printables)), joinString=" ", adjacent=False
)
ddline = White(min=0).suppress() + restOfLine + LineEnd()
dedented = Combine(OneOrMore(ddline))
norm_inlinewhite = OneOrMore(" \t").setParseAction(replaceWith(" "))
sup_hyphen = Suppress("-")
sup_colon = Suppress(":")
sup_semicolon = Suppress(";")
sup_comma = Suppress(",")
sup_lparen = Suppress("(")
sup_rparen = Suppress(")")
sup_space = Suppress(" ")
matchall = Regex("(.|\n)*")

accession = Word(alphas, bodyChars=alphanums + "_")
gi = Suppress("GI:") + Word(nums)

# Locus Parsing
name = Word(printables)("name")
length = Word(nums)("length").setParseAction(to_int)
base_type = (Literal("bp") ^ Literal("aa"))("base_type")
strandedness = Optional(Or(map(Literal, STRANDS)))("strandedness")
mol_type = Optional(Or(map(Literal, MOL_TYPES)))("molecule_type")
topology = Optional(Or(map(Literal, TOPOLOGY)))("topology")
division = Or(map(Literal, DIVISIONS))("division")
day = Word(nums, exact=2).setParseAction(to_int)("day")
month = Word(alphas, exact=3).setParseAction(to_month)("month")
year = Word("12", bodyChars=nums, exact=4).setParseAction(to_int)("year")
date = Group(day + sup_hyphen + month + sup_hyphen + year)("date")
date = date.setParseAction(to_date)
locus = LineStart() + Literal("LOCUS").suppress() + name + length \
        + base_type + strandedness + mol_type + topology + division + date

# Keyword Parsing
firstline = WordStart() + restOfLine + LineEnd()
continuation = White() + firstline
keywordname = Word(alphas + "_") + White()
keyword = Combine(
    keywordname + firstline + ZeroOrMore(continuation), adjacent=False
)
keywordsection = OneOrMore(keyword)

optspace = Optional(sup_space)
keywordfirst = ZeroOrMore(Word(printables) + optspace) + LineEnd().suppress()
keywordcont = White(exact=12).suppress() + keywordfirst
keywordnorm = Combine(keywordfirst + ZeroOrMore(keywordcont), joinString=" ")

# DEFINITION
definition = Literal("DEFINITION").suppress() + normalized

# ACCESSION
accfirst = accession.setResultsName("first")
acclast = accession.setResultsName("last")
accrange = Group(accfirst + sup_hyphen + acclast)
accession = Literal("ACCESSION").suppress() + OneOrMore(accrange ^ accession)

# VERSION
veracc = Combine(
    Word(alphanums + "_") + Optional("." + Word(nums))("accession")
)
version = Literal("VERSION").suppress() + veracc + gi

# PID
pid = Literal("PID").suppress() + Word(alphanums)

# DBSOURCE
dbsource = Literal("DBSOURCE").suppress() + dedented

# DBLINK
dbln_type = Or(map(Literal, ["Project", "Trace Assembly Archive"]))
dbln_type = dbln_type.setParseAction(lambda t: t[0].lower())
dbln_list = Group(delimitedList(Word(alphanums + "_")))
dblink = Literal("DBLINK") + Group(dbln_type + sup_colon + dbln_list)

# PROJECT
project = Literal("PROJECT").suppress() + normalized

# KEYWORDS
kwphrase = ZeroOrMore(Word(''.join(p for p in printables if p != ";")))
kwphrase = Combine(kwphrase, joinString=" ", adjacent=False)
kwphrases = delimitedList(kwphrase, delim=";")
kwempty = Suppress("." + LineEnd())
keywords = Literal("KEYWORDS").suppress() + (kwempty | kwphrases)

# SEGMENT
segment = Literal("SEGMENT").suppress() \
            + integer("position") + Suppress("of") + integer("total")

# SOURCE
sourcename = Literal("SOURCE").suppress() + keywordnorm
organism = Literal("ORGANISM").suppress() + Combine(WordStart() + restOfLine)
taxname = Combine(OneOrMore(
    Word(filter(lambda p: p != ";", printables)) + Optional(sup_space)
), joinString=" ", adjacent=True)
taxonomy = delimitedList(OneOrMore(taxname), delim=";")
source = sourcename + organism + taxonomy

# REFERENCE
basesorresidues = Suppress("bases") | Suppress("residues")
position = basesorresidues + integer("from") + Suppress("to") + integer("to")
notes = Word(''.join(p for p in printables if p not in "()"))("notes")
refloc = sup_lparen + Optional(position) + Optional(notes) + sup_rparen
refdesc = integer("refid") + Optional(Group(refloc))
authors = Group(Literal("AUTHORS") + keywordnorm)
consrtm = Group(Literal("CONSRTM") + keywordnorm)
title = Group(Literal("TITLE") + keywordnorm)
journal = Group(Literal("JOURNAL") + keywordnorm)
medline = Group(Literal("MEDLINE") + Word(nums, exact=8)("medline"))
pubmed = Group(Literal("PUBMED") + Word(nums)("pubmed"))
remark = Group(Literal("REMARK") + keywordnorm)
refsubs = OneOrMore(
    authors ^ consrtm ^ title ^ journal ^ medline ^ pubmed ^ remark
)
reference = Literal("REFERENCE").suppress() + refdesc + refsubs

# PRIMARY
priheaderliterals = "REFSEQ_SPAN PRIMARY_IDENTIFIER PRIMARY_SPAN COMP".split()
priheader = And(map(Literal, priheaderliterals)).suppress()
pri_span = Group(integer("from") + sup_hyphen + integer("to"))
pri_row = Group(pri_span + veracc + pri_span + Optional("c"))
primary = Literal("PRIMARY").suppress() + priheader + OneOrMore(pri_row)

# COMMENT
comment = Literal("COMMENT").suppress() + dedented

# LOCATION
location = Forward()
args = (sup_lparen + delimitedList(location) + sup_rparen)
index = Group(Optional(Word("<>", exact=1)) + integer)
intargs = (sup_lparen + delimitedList(integer) + sup_rparen)
bond = Group(Literal("bond") + intargs)("bond")
one_of = Group(Literal("one-of") + intargs)("one-of")
position = index | one_of
span = Group(position + Suppress("..") + position)
site = Group(index + Suppress("^") + index)
choice = Group(index + Suppress(".") + index)
gap = Group(Literal("gap") + sup_lparen + integer + sup_rparen)
complement = Group(Literal("complement") + args)
join = Group(Literal("join") + args)("join")
order = Group(Literal("order") + args)("order")
referenceloc = Group(veracc + sup_colon + location)("reference")
location << (
    span | complement | join | order | bond | gap | referenceloc \
        | site | choice | index | one_of
)

# FEATURES
#
# Hand coded feature spliting to speed up parsing.
#
# featcont = White(exact=21).suppress() + restOfLine + LineEnd()
# feature = Combine(
#    White(exact=5).suppress() + restOfLine + LineEnd() + ZeroOrMore(featcont),
#     adjacent=False
# )
# featuresection = OneOrMore(feature)

# featname = Word(alphanums + "_'")
# featlocword = Word(''.join(filter(lambda p: p != "/", printables)))
# featloc = Combine(OneOrMore(featlocword), adjacent=False)
# qualname = Word(''.join(filter(lambda p: p != "=", printables)))
# qualquoted = Combine(
#     Suppress('"') + Regex(r'[^"]*') + Suppress('"'),
#     adjacent=True
# )
# qualunquoted = Word(filter(lambda p: p != '"', printables))
# qualvalue = Suppress("=") + (qualquoted | qualunquoted) + LineEnd().suppress()
# qualnovalue = LineEnd().suppress()
# qualifier = Group(Suppress("/") + qualname + (qualvalue | qualnovalue))
# featgroup = featname + featloc + ZeroOrMore(qualifier)

# CONTIG
contig = Literal("CONTIG") + location

# BASE COUNT
single_count = Group(integer + Word(alphas))
base_count = Literal("BASE COUNT") + OneOrMore(single_count)

# TRANSL_EXCEPT
#(pos:11930596..11930598,aa:Asp)
mod_pos = Literal("pos").suppress() + sup_colon + location
mod_acid = Literal("aa").suppress() + sup_colon + Word(alphas)
modification = sup_lparen + mod_pos + sup_comma + mod_acid + sup_rparen


#
# Keyword Parsers
#

def _k_definition(data):
    return definition.parseString(data)[0].rstrip(".")

def _k_accession(data):
    res = accession.parseString(data)
    ret = {"primary": res[0], "secondary": []}
    for acc in res[1:]:
        if hasattr(acc, "first"):
            ret["secondary"].append({"first": acc.first, "last": acc.last})
        else:
            ret["secondary"].append(acc)
    return ret

def _k_version(data):
    res = version.parseString(data)
    return {"accession": res[0], "gi": res[1]}

def _k_pid(data):
    return pid.parseString(data)[0]

def _k_dbsource(data):
    return dbsource.parseString(data)[0].strip()

def _k_dblink(data):
    ret = []
    res = dblink.parseString(data)
    for group in res:
        ret.append({
            "type": group[0],
            "accessions": group[1]
        })
    return ret

def _k_segment(data):
    res = segment.parseString(data)
    return {"position": res.position, "of": res.total}

def _k_project(data):
    return project.parseString(data)[0]

def _k_keywords(data):
    res = keywords.parseString(data)
    return [kw.rstrip(".") for kw in res]

def _k_source(data):
    res = source.parseString(data)
    ret = {"name": res[0].rstrip(".")}
    if len(res) > 1:
        ret["organism"] = res[1]
        ret["taxonomy"] = [r.rstrip(".") for r in res[2:]]
    return ret

def _k_reference(data):
    res = reference.parseString(data)
    loc = {
        "from": getattr(res[1], "from", None),
        "to": getattr(res[1], "to", None)
    }
    if not loc["from"] and not loc["to"]:
        loc = None
    ref = {
        "refid": res[0],
        "location": loc,
        "notes": getattr(res[1], "notes", "")
    }
    for r in res[2:]:
        if r[0].lower() in ref:
            raise ParseException(data, len(data), "Repeated subkeyword.")
        ref[r[0].lower()] = r[1]
    return ref

def _k_primary(data):
    res = primary.parseString(data.strip())
    ret = []
    for row in res:
        if len(row) < 3 or len(row) > 4:
            raise ParseException(data, len(data), "Invalid PRIMARY row.")
        ret.append({
            "refseq_span": {"from": row[0]["from"], "to": row[0]["to"]},
            "primary_identifier": row[1],
            "primary_span": {"from": row[2]["from"], "to": row[2]["to"]},
            "complement": (len(row) == 4 and row[3] == "c")
        })
    return ret

def _k_comment(data):
    return comment.parseString(data)[0].strip()



#
# Location Parsers
#

def _l_index(t):
    return {
        "type": "index",
        "position": t[0][-1],
        "fuzzy": str(t[0][0]) in "<>"
    }

def _l_bond(t):
    return {
        "type": "bond",
        "positions": t[0][1:]
    }

def _l_one_of(t):
    return {
        "type": "one-of",
        "positions": t[0][1:]
    }

def _l_span(t):
    if t[0][0]["type"] == "index":
        fr = t[0][0]["position"]
    else:
        fr = t[0][0]
    if t[0][1]["type"] == "index":
        to = t[0][1]["position"]
    else:
        to = t[0][1]
    ret = {"type": "span", "from": fr, "to": to}
    if t[0][0].get("fuzzy"):
        ret.setdefault("modifiers", {})
        ret["modifiers"]["from"] = "fuzzy"
    if t[0][1].get("fuzzy"):
        ret.setdefault("modifiers", {})
        ret["modifiers"]["to"] = "fuzzy"
    return ret

def _l_site(t):
    return {
        "type": "site",
        "between": t[0][:]
    }

def _l_choice(t):
    return {
        "type": "choice",
        "between": t[0][:]
    }

def _l_gap(t):
    return {
        "type": "gap",
        "length": t[0][1]
    }

def _l_complement(t):
    return {
        "type": "complement",
        "segment": t[0][1]
    }

def _l_join(t):
    return {
        "type": "join",
        "segments": t[0][1:]
    }

def _l_order(t):
    return {
        "type": "order",
        "segments": t[0][1:]
    }

def _l_referenceloc(t):
    return {
        "type": "reference",
        "accession": t[0][0],
        "segment": t[0][1]
    }

def _mk_locs():
    lnames = """
        index bond one_of span site choice gap
        complement join order referenceloc
    """.split()
    for ln in lnames:
        elem = globals()[ln].setParseAction(globals()["_l_%s" % ln])
        globals()[ln] = elem
_mk_locs()



#
# Qualifier Parsers
#
_qual_default = lambda d: ' '.join(d.strip('"').split())

def _q_transl_except(d):
    res = modification.parseString(d)
    return {"location": res[0], "acid": res[1]}

_q_anticodon = _q_transl_except
_q_codon_start = lambda d: int(d)
_q_number = lambda d: int(d)
_q_pseudo = lambda d: True
_q_ribosomal_slippage = lambda d: True
_q_selenocysteine = lambda d: True
_q_translation = lambda d: ''.join(d.strip('"').split())
_q_virion = lambda d: True

known_qualifiers = set("""
allele bond_type chromosome citation clone clone_lib coded_by codon_recognized
country cultivar db_xref dev_stage direction ec_number evidence exception
experiment function gene gene_synonym go_component go_function go_process host
inference isolate locus_tag map mobile_element mol_type note ncrna_class
old_locus_tag operon organelle organism product protein_id region_name
rpt_family site_type standard_name strain sub_strain tissue_type transcript_id
transl_table
""".split())

def _add_qualifier(s, quals, name, data):
    if name not in MULTI_QUALIFIERS:
        if name in quals:
            warnings.warn("Unexpected qualifier repetition: %s" % name)
        else:
            quals[name] = data
            return
    if name not in quals:
        quals[name] = []
    elif name in quals and not isinstance(quals[name], list):
        quals[name] = [quals[name]]
    quals[name].append(data)


#
# Public parsing methods
#

def parse_locus(data):
    return locus.parseString(data).asDict()

def parse_keyword(data):
    name = data.split(None, 1)[0].lower()
    try:
        func = globals()["_k_%s" % name]
        return func(data)
    except KeyError:
        warnings.warn("No keyword parser for: %r" % name)
        return data

def parse_keywords(data):
    ret = {}
    res = keywordsection.parseString(data)
    for kw in res:
        name = kw.split(None, 1)[0].lower()
        kwdata = parse_keyword(kw)
        if name not in MULTI_KEYWORDS:
            if name in ret:
                raise KeyError("Keyword '%s' is not repeatable." % name)
            ret[name] = kwdata
        else:
            if name not in ret:
                ret[name] = []
            ret[name].append(kwdata)
    return ret

def parse_location(data):
    return location.parseString(data)[0]

def parse_qual_name(data, pos):
    while pos < len(data) and data[pos] == " ":
        pos += 1
    if data[pos] != "/":
        raise ParseException(data, len(data), "Incorrect qualifier name start.")
    for i in range(pos+1, len(data)):
        if data[i] == "=" or data[i] == " ":
            return (data[pos+1:i], i)
    return (data[pos+1:], len(data))

def parse_qual_value(data, pos):
    if pos >= len(data):
        return (None, pos)
    if data[pos] == " ":
        return (None, pos+1)
    if data[pos] != "=":
        raise ParseException(data, len(data), "Incorrect qualifiver val start.")
    if data[pos+1] == '"':
        off, delim = 1, '"'
    else:
        off, delim = 0, ' '
    for i in range(pos+1+off, len(data)):
        if data[i] == delim:
            return (data[pos+1:i+off], i+off)
    return (data[pos+1:], len(data))

def parse_qualifiers(data):
    if not data:
        return
    pos = 0
    while pos < len(data):
        (qname, pos) = parse_qual_name(data, pos)
        (qval, pos) = parse_qual_value(data, pos)
        yield (qname, qval)

FEAT_GROUP = re.compile("^(\S+)\s+([^/]+)(?:\s+(.*))?$")
def parse_feature(data):
    data = ' '.join(data.split())
    match = FEAT_GROUP.match(data)
    if not match:
        raise ValueError(data)
    feattype = match.group(1)
    featloc = location.parseString(match.group(2))
    props = {}
    glbls = globals()
    for (qname, qval) in parse_qualifiers(match.group(3)):
        qname, qval = qname.lower(), qval.strip('"') if qval else None
        func = glbls.get("_q_%s" % qname)
        if func is None:
            if qname not in known_qualifiers:
                warnings.warn("No parser for qualifier: %s" % qname)
            _add_qualifier(data, props, qname, qval)
        else:
            _add_qualifier(data, props, qname, func(qval))
    return {
        "type": match.group(1),
        "location": parse_location(match.group(2)),
        "properties": props
    }
    #res = featgroup.parseString(data)
    #props = {}
    #for q in res[2:]:
    #    qname = q[0].lower()
    #    value = q[1] if len(q) > 1 else None
    #    func = globals().get("_q_%s" % qname)
    #    if func is None:
    #        warnings.warn("No parser for qualifier: %s" % qname)
    #        _add_qualifier(data, props, qname, value)
    #    else:
    #        _add_qualifier(data, props, qname, func(value))
    #return {
    #    "type": res[0],
    #    "location": parse_location(res[1]),
    #    "properties": props
    #}

NEXT_FEAT = re.compile("^\s{5}\S")
def parse_features(data):
    ret = []
    curr = []
    for line in Reader.from_string(data):
        if NEXT_FEAT.match(line):
            if len(curr) > 0:
                ret.append(parse_feature(''.join(curr)))
            curr = [line]
        else:
            curr.append(line)
    if len(curr):
        ret.append(parse_feature(''.join(curr)))
    return ret
    #for feat in featuresection.parseString(data):
    #    ret.append(parse_feature(feat))
    #return ret

def parse_contig(data):
    return contig.parseString(data)[0]

def parse_base_count(data):
    res = base_count.parseString(data)
    return dict((g[1].upper(), g[0]) for g in res)

class Reader(object):
    def __init__(self, stream):
        self.stream = itertools.ifilter(Reader.skipblank, stream)

    def __iter__(self):
        return self

    @staticmethod
    def skipblank(line):
        return bool(line.strip())

    @staticmethod
    def from_string(strdata):
        def _iter(data):
            prev = 0
            next = data.find("\n")
            while next >= 0:
                yield data[prev:next+1]
                prev, next = next+1, data.find("\n", next+1)
            yield data[prev:]
        return Reader(_iter(strdata))

    def next(self):
        return self.stream.next()
    
    def read_to(self, marker):
        lines = []
        for line in self.stream:
            if line.startswith(marker):
                return (''.join(lines), line)
            if line.startswith("//"):
                raise ValueError("Read past record separator.")
            lines.append(line)
        return (''.join(lines), None)

class Record(object):
    def __init__(self, stream):
        if not isinstance(stream, Reader):
            stream = Reader(iter(stream))
        
        self.locus = parse_locus(stream.next())
        self.keywords = parse_keywords(stream.read_to("FEATURES")[0])

        (data, mark) = stream.read_to(("BASE COUNT", "ORIGIN", "CONTIG"))
        self.features = parse_features(data)

        self.base_count = None
        self.contig = None
        self.seqiter = None
        
        if mark.startswith("BASE COUNT"):
            self.base_count = parse_base_count(mark)
            (data, mark) = stream.read_to(("CONTIG", "ORIGIN"))
        
        if mark.startswith("CONTIG"):
            data = ''.join([mark, stream.read_to("//")[0]])
            self.contig = parse_contig(data)
        else:
            self.seqiter = Sequence(stream)

    def __repr__(self):
        return "<Record %r>" % self.locus["name"]

    def load_sequence(self):
        self.sequence = ''.join(self.seqiter)
        self.seqiter = None

    def exhaust(self):
        if not self.seqiter:
            return
        for seq in self.seqiter:
            pass

class Sequence(object):
    def __init__(self, stream):
        self.exhausted = False
        self.stream = stream

    def __iter__(self):
        return self

    def next(self):
        if self.exhausted:
            raise StopIteration
        line = self.stream.next()
        if line.strip() == "//":
            self.exhausted = True
            raise StopIteration
        return ''.join(line.split()[1:])


#
# Main Entry Points
#

def parse(data):
    if isinstance(data, types.StringTypes):
        stream = Reader.from_string(data)
    else:
        stream = Reader(iter(data))
    while True:
        rec = Record(stream)
        yield rec
        rec.exhaust()

def parse_file(filename):
    # Round about way to make sure the file
    # stays open for the lifetime of the record.
    def _stream(fname):
        with open(fname) as handle:
            for line in handle:
                yield line
    for r in parse(_stream(filename)):
        yield r



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import os
import re
import types
import warnings

import t

TESTRE = re.compile(r'test')
def get_checks(idx, fname):
    modname = "regressions.%s-%d" % (fname, idx)
    try:
        mod = __import__(modname)
    except Exception, inst:
        warnings.warn("Unable to find checks for %s:\n%s" % (fname, inst))
        return
    mod = getattr(mod, "%s-%d" % (fname, idx))
    for funcname in dir(mod):
        if not TESTRE.search(funcname):
            continue
        attr = getattr(mod, funcname)
        if not isinstance(attr, types.FunctionType):
            continue
        attr.func_name = "regression.%s.%d" % (fname, idx)
        yield attr

def test_regressions():
    dirname = os.path.dirname(__file__)
    datapath = os.path.join(dirname, "data")
    for path, dnames, fnames in os.walk(datapath):
        fnames = filter(lambda x: x.endswith(".gb"), fnames)
        for fname in fnames:
            gbfname = os.path.join(path, fname)
            for idx, rec in enumerate(t.nebgb.parse_file(gbfname)):
                for check in get_checks(idx, fname[:-3]):
                    yield check, rec
        
        
        


#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgb package released under the MIT license.
#

import t

@t.parse("parse_keywords")
def test_1():
    """
DEFINITION  Pseudomonas syringae pv. averrhoi strain HL1 leucyl-tRNA synthetase
            (leuS) gene, partial cds; putative lipoprotein, DNA polymerase III
            delta subunit (holA), type III effector HopAJ2 (hopAJ2), lipoic
            acid synthetase (lipA), and lipoate-protein ligase B (lipB) genes,
            complete cds; and hypothetical protein gene, partial cds.
ACCESSION   EU143250
VERSION     EU143250.1  GI:157644754
KEYWORDS    .
SOURCE      Pseudomonas syringae pv. averrhoi
  ORGANISM  Pseudomonas syringae pv. averrhoi
            Bacteria; Proteobacteria; Gammaproteobacteria; Pseudomonadales;
            Pseudomonadaceae; Pseudomonas.
REFERENCE   1  (bases 1 to 7304)
  AUTHORS   Kuo,C.-H., Lin,Y.-C. and Huang,H.-C.
  TITLE     Characterization of hopAJ1 and hopAJ2 genes from Pseudomonas
            syringae pv. averrhoi
  JOURNAL   Unpublished
REFERENCE   2  (bases 1 to 7304)
  AUTHORS   Kuo,C.-H., Lin,Y.-C. and Huang,H.-C.
  TITLE     Direct Submission
  JOURNAL   Submitted (07-SEP-2007) Graduate Institute of Biotechnology,
            National Chung Hsing University, 250 Kuo-Kuang Rd., Taichung 402,
            Taiwan
    """
    return {
        "definition": t.normalize("""
            Pseudomonas syringae pv. averrhoi strain HL1 leucyl-tRNA synthetase
            (leuS) gene, partial cds; putative lipoprotein, DNA polymerase III
            delta subunit (holA), type III effector HopAJ2 (hopAJ2), lipoic
            acid synthetase (lipA), and lipoate-protein ligase B (lipB) genes,
            complete cds; and hypothetical protein gene, partial cds       
        """),
        "accession": {"primary": "EU143250", "secondary": []},
        "version": {"accession": "EU143250.1", "gi": "157644754"},
        "keywords": [],
        "source": {
            "name": "Pseudomonas syringae pv. averrhoi",
            "organism": "Pseudomonas syringae pv. averrhoi",
            "taxonomy": """
                Bacteria Proteobacteria Gammaproteobacteria Pseudomonadales
                Pseudomonadaceae Pseudomonas
            """.split()
        },
        "reference": [{
            "refid": 1,
            "location": {"from": 1, "to": 7304},
            "notes": "",
            "authors": "Kuo,C.-H., Lin,Y.-C. and Huang,H.-C.",
            "title": t.normalize("""
                Characterization of hopAJ1 and hopAJ2 genes from Pseudomonas
                syringae pv. averrhoi
            """),
            "journal": "Unpublished"
        },{
            "refid": 2,
            "location": {"from": 1, "to": 7304},
            "notes": "",
            "authors": "Kuo,C.-H., Lin,Y.-C. and Huang,H.-C.",
            "title": "Direct Submission",
            "journal": t.normalize("""
                Submitted (07-SEP-2007) Graduate Institute of Biotechnology,
                National Chung Hsing University, 250 Kuo-Kuang Rd., Taichung 402,
                Taiwan
            """)
        }]
    }